﻿Imports dataq.simple

Public Class frmDI_245Example
    Dim DataqDeviceArray() As DataqDevice 'array containing all connected devices
    Dim WithEvents DI_245 As DataqDevice 'this is the primary device we'll work with
    Dim CollectedData(11) As Double 'dimension the array that will hold acquired data
    Dim ChannelConfig(3) As String 'array to retain the channel configuration defined via the GUI


    Private Sub frmDI_245Example_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        'upon loading the application do some device discovery, connection, and housekeeping

        'Discover connected devices
        DataqDeviceArray = Discovery.DiscoverAllDevices

        'Use only the first device in the discovery array
        'Bail if the first arrary position is empty
        If DataqDeviceArray.Length = 0 Then
            MessageBox.Show("No compatible DATAQ Instruments devices found.", _
                            "Missing DATAQ Instruments hardware", _
                            MessageBoxButtons.OK, _
                            MessageBoxIcon.Error)
            End

            'get here if the first array position contains something
        Else
            DI_245 = DataqDeviceArray(0) 'assign DataqDevice DI_245 to the first device in the array and...
            DI_245.Connect() '...connect to it

        End If

        'display model-specific information
        lblModel.Text = DI_245.Model
        lblSerial.Text = DI_245.SerialNumber
        lblPort.Text = DI_245.ComPort

        'Bail if the connected device is not model DI-245
        If DI_245.Model <> "DI-245" Then
            MessageBox.Show("This program works only with DATAQ Instruments model DI-245", _
                           "Incompatible Device (" + DI_245.Model + ")", _
                           MessageBoxButtons.OK, _
                           MessageBoxIcon.Error)
            End
        End If

        'load up ChannelConfig list with a default analog channel configuration
        For i As Integer = 0 To 3
            ChannelConfig(i) = "1" 'set all analog channels to the ±1 V full scale range
        Next
        cbChannelConfig.SelectedIndex = 15 'display ±1 V fs to be consistent

        'everthing's good, so create an event handler that fires when new DI-245 data is available
        AddHandler DI_245.NewData, AddressOf GetDI245Data
    End Sub
    Private Sub GetDI245Data()
        'this is the event handler for DI-245 data. Get here when new data is available during a scan
        Dim Scans As Integer = DI_245.NumberOfScansAvailable
        Dim Channels As Short = DI_245.NumberOfChannelsEnabled
        Dim DI_245_Data(Scans * Channels - 1) As Double 'will hold all data for the scan
        Dim ResponseString As String = ""

        'Move data into temporary array
        DI_245.GetInterleavedScaledData(DI_245_Data, 0, Scans)
        'now move it to the list box
        For Row As Integer = 0 To Scans - 1
            For Column As Short = 0 To Channels - 1
                ResponseString = ResponseString + DI_245_Data(Column + (Row * Channels)).ToString("F3")
                If Column <> Channels - 1 Then
                    ResponseString = ResponseString + ", "
                End If
            Next
            UpdateList(ResponseString + vbCrLf) 'output to the list box
            ResponseString = "" 'reset ResponseString
        Next
    End Sub
    Private Sub btnStart_Click(sender As System.Object, e As System.EventArgs) Handles btnStart.Click
        'this sub configures the DI-245 scan list. It uses array ChannelConfig() to configure
        'analog channels, and makes one final, separate test to determine if digital input data is 
        'active. 

        Dim VoltageRange As New dataq.Range(Of Double) 'VoltageRange holds the FSR of a voltage channel
        'first, some GUI housekeeping
        btnStart.Enabled = False
        btnStop.Enabled = True
        gbChannelConfig.Enabled = False
        cbSampleRate.Enabled = False
        lbOutput.Items.Clear()
        lblSamplingRate.Text = ""

        'Configure the DI-245's scan list for analog channels
        'do the following for each of the four analog channels:
        For Channel As Integer = 0 To 3
            'first reset channel to TC and disable it to make any new configuration less complex
            DI_245.ChangeChannelType(Channel, ChannelType.Thermocouple)
            DI_245.ChannelArray(Channel).Enabled = False

            'now configure the channel as defined by array ChannelConfig
            Select Case True
                Case ChannelConfig(Channel).StartsWith("J")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.J
                Case ChannelConfig(Channel).StartsWith("K")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.K
                Case ChannelConfig(Channel).StartsWith("T")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.T
                Case ChannelConfig(Channel).StartsWith("B")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.B
                Case ChannelConfig(Channel).StartsWith("R")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.R
                Case ChannelConfig(Channel).StartsWith("S")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.S
                Case ChannelConfig(Channel).StartsWith("E")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.E
                Case ChannelConfig(Channel).StartsWith("N")
                    DI_245.ChannelArray(Channel).TCType = TCTypeName.N
                Case ChannelConfig(Channel) = "Off"
                    'the channel is already off, so we don't need to do anything more
                Case Else
                    'get here if a voltage config is specified
                    VoltageRange.Maximum = ChannelConfig(Channel)
                    DI_245.ChangeChannelType(Channel, ChannelType.AnalogVoltage) 'set channel type to analog voltage
                    DI_245.ChannelArray(Channel).InputRange = VoltageRange 'and set the range
            End Select
            If ChannelConfig(Channel) <> "Off" Then
                'in addition to configuring a channel, also need to enable it
                DI_245.ChannelArray(Channel).Enabled = True
            End If
        Next

        'now set up the digital input channel
        If cbDigIn.Checked Then
            DI_245.ChannelArray(8).Enabled = True
        Else
            DI_245.ChannelArray(8).Enabled = False
        End If

        'set the sampling rate
        'Note: the class will always select the sampling rate closest to the one selected constrained
        'by the number of enabled channels. For a full treatment of how to configure DI-245 sampling rate
        'refer to the instrument's prototocol document:
        'http://www.dataq.com/resources/pdfs/support_articles/DI-245-protocol.pdf
        DI_245.SampleRatePerChannel = Val(cbSampleRate.Text)

        'display the actual sampling rate per channel
        lblSamplingRate.Text = DI_245.SampleRatePerChannel.ToString("F3")

        DI_245.NewDataMinimum = 1 'set the number of scans to acquire before the NewData event fires
        DI_245.Start() 'start scanning
    End Sub
    Private Sub cbChannelConfig_SelectedIndexChanged(sender As System.Object, e As System.EventArgs) Handles cbChannelConfig.SelectedIndexChanged
        'When combobox cbChannelConfig changes, update array ChannelConfig accordingly
        'as a function of the enabled channel
        Select Case True
            Case rbChan1.Checked
                ChannelConfig(0) = cbChannelConfig.Text
            Case rbChan2.Checked
                ChannelConfig(1) = cbChannelConfig.Text
            Case rbChan3.Checked
                ChannelConfig(2) = cbChannelConfig.Text
            Case rbChan4.Checked
                ChannelConfig(3) = cbChannelConfig.Text
        End Select
    End Sub
    Private Sub rbChan1_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbChan1.CheckedChanged
        'diplay the configuration of analog channel 1 in combo box cbChannelConfig
        If rbChan1.Checked Then
            cbChannelConfig.Text = ChannelConfig(0)
        End If
    End Sub
    Private Sub rbChan2_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbChan2.CheckedChanged
        'diplay the configuration of analog channel 2 in combo box cbChannelConfig
        If rbChan2.Checked Then
            cbChannelConfig.Text = ChannelConfig(1)
        End If
    End Sub
    Private Sub rbChan3_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbChan3.CheckedChanged
        'diplay the configuration of analog channel 3 in combo box cbChannelConfig
        If rbChan3.Checked Then
            cbChannelConfig.Text = ChannelConfig(2)
        End If
    End Sub
    Private Sub rbChan4_CheckedChanged(sender As System.Object, e As System.EventArgs) Handles rbChan4.CheckedChanged
        'diplay the configuration of analog channel 4 in combo box cbChannelConfig
        If rbChan4.Checked Then
            cbChannelConfig.Text = ChannelConfig(3)
        End If
    End Sub
    Private Sub btnStop_Click(sender As System.Object, e As System.EventArgs) Handles btnStop.Click
        'stop button clicked so stop the DI-245 and perform some GUI housekeeping
        DI_245.Stop()
        btnStart.Enabled = True
        btnStop.Enabled = False
        gbChannelConfig.Enabled = True
        cbSampleRate.Enabled = True
    End Sub
    Private Sub btnClear_Click(sender As System.Object, e As System.EventArgs) Handles btnClear.Click
        'Clear the output list box
        lbOutput.Items.Clear()
    End Sub

    'UpdateList is called from a different thread than the main program, so we need to delegate
    Private Delegate Sub UpdateListDelegate(ByVal ResponseString As String) 'create new class UpdateListDelegate
    Private Sub UpdateList(ByVal ResponseString As String)
        If Me.InvokeRequired Then
            Me.Invoke(New UpdateListDelegate(AddressOf UpdateList), ResponseString) 'invoke delegate then...
        Else
            lbOutput.Items.Add(ResponseString) '...output string to list box
        End If
    End Sub

End Class
